# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for redfish_json_validator.py
"""

import collections
import json
from unittest import mock

from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.errors \
    import OneViewRedfishResourceNotFoundError
from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator
from oneview_redfish_toolkit.tests.base_test import BaseTest

service_root_version = 'v1_2_0'
zone_version = 'v1_1_0'

schemas_dict = collections.OrderedDict()
schemas_dict['ServiceRoot'] = 'ServiceRoot.' + service_root_version + '.json'
schemas_dict['Zone'] = 'Zone.' + zone_version + '.json'


class TestRedfishJsonValidator(BaseTest):

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated
        try:
            redfish_json_validator = RedfishJsonValidator('ServiceRoot')
        except Exception as e:
            self.fail("Failed to instantiate RedfishJsonValidator class."
                      " Error: {}".format(e))
        self.assertIsInstance(redfish_json_validator, RedfishJsonValidator)

    def test_has_valid_config_file(self):
        # Tests if expected filed exists and are correctly populated by
        # the constructor

        redfish_json_validator = RedfishJsonValidator('ServiceRoot')
        self.assertIsInstance(
            redfish_json_validator.redfish,
            collections.OrderedDict)

    def test_get_resource_by_id(self):
        redfish_json_validator = RedfishJsonValidator('ServiceRoot')

        # Loading server_hardware mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            server_hardware = json.load(f)

        device_slot = redfish_json_validator.get_resource_by_id(
            server_hardware["portMap"]["deviceSlots"], "deviceNumber", 3)

        json_device_slot = None

        for i in server_hardware["portMap"]["deviceSlots"]:
            if i["deviceNumber"] == 3:
                json_device_slot = i

        self.assertEqual(device_slot, json_device_slot)

    def test_get_resource_empty_list(self):
        redfish_json_validator = RedfishJsonValidator('ServiceRoot')

        with self.assertRaises(OneViewRedfishResourceNotFoundError):
            redfish_json_validator.get_resource_by_id([], "deviceNumber", 1)

    def test_get_resource_invalid_id(self):
        redfish_json_validator = RedfishJsonValidator('ServiceRoot')

        with self.assertRaises(OneViewRedfishError):
            redfish_json_validator.get_resource_by_id(
                [], "deviceNumber", "INVALID_ID")

    @mock.patch('oneview_redfish_toolkit.api.schemas.SCHEMAS', schemas_dict)
    def test_get_odata_type_by_class(self):
        redfish_json_validator = RedfishJsonValidator('ServiceRoot')
        odata_type_schema_class = '#ServiceRoot.' + \
            service_root_version + '.ServiceRoot'

        zone_schema_name = 'Zone'
        odata_type_zone_schema = '#Zone.' + zone_version + '.Zone'

        self.assertEqual(redfish_json_validator.get_odata_type(),
                         odata_type_schema_class)
        self.assertEqual(
            redfish_json_validator.get_odata_type_by_schema(zone_schema_name),
            odata_type_zone_schema)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection
from oneview_redfish_toolkit.api import status_mapping


class StorageCompositionDetails(RedfishJsonValidator):
    """Creates a StorageCompositionDetails dict

            Populates self.redfish with data retrieved from
            an OneView's Drive
    """

    SCHEMA_NAME = 'Storage'

    def __init__(self, drive):
        """StorageCompositionDetails constructor

            Populates self.redfish with the contents of devices from an
            Oneview's Drive

            Args:
                drive: Oneview's Drive dict
        """
        super().__init__(self.SCHEMA_NAME)

        drive_uuid = drive["uri"].split("/")[-1]
        self.odata_id = "{}/{}/Storage/1"\
            .format(ResourceBlockCollection.BASE_URI, drive_uuid)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = "1"
        self.redfish["Name"] = drive["name"]
        self.redfish["Status"] = status_mapping.STATUS_MAP.get(drive["status"])
        self.redfish["Drives"] = [
            {
                "@odata.id": self.odata_id + "/Drives/1"
            }
        ]
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#Storage.Storage"

        self.redfish["@odata.id"] = self.odata_id

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy
import json

from oneview_redfish_toolkit.api.blade_chassis import BladeChassis
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestBladeChassis(BaseTest):
    """Tests for Chassis class"""

    def setUp(self):
        """Tests preparation"""

        # Loading server_hardware mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        # Loading Chassis mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/BladeChassis.json'
        ) as f:
            self.blade_chassis_mockup = json.load(f)

        self.manager_uuid = "b08eb206-a904-46cf-9172-dcdff2fa9639"

    def test_serialize_when_blade_chassis_has_computer_system(self):
        blade_chassis = BladeChassis(
            self.server_hardware, self.manager_uuid
        )

        result = json.loads(blade_chassis.serialize())

        self.assertEqualMockup(self.blade_chassis_mockup, result)

    def test_serialize_when_blade_chassis_has_not_computer_system(self):
        server_hardware = copy.deepcopy(self.server_hardware)
        server_hardware["serverProfileUri"] = None

        blade_chassis = BladeChassis(
            server_hardware, self.manager_uuid
        )
        result = json.loads(blade_chassis.serialize())

        expected_blade_result = copy.deepcopy(self.blade_chassis_mockup)
        expected_blade_result["Links"]["ComputerSystems"] = []

        self.assertEqualMockup(expected_blade_result, result)

        server_hardware["serverProfileUri"] = ""

        blade_chassis = BladeChassis(
            server_hardware, self.manager_uuid
        )
        result = json.loads(blade_chassis.serialize())

        self.assertEqualMockup(expected_blade_result, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import json

from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestResourceBlockCollection(BaseTest):
    """Tests for ResourceBlockCollection class"""

    def setUp(self):
        """Tests preparation"""

        # Loading ServerHardware list mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerHardwareList.json'
        ) as f:
            self.server_hardware_list = json.load(f)

        # Loading ServerProfileTemplate list mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerProfileTemplates.json'
        ) as f:
            self.server_profile_template_list = json.load(f)

        # Loading Drives list mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/Drives.json'
        ) as f:
            self.drives_list = json.load(f)

        # Loading ResourceBlockCollection result mockup
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'ResourceBlockCollection.json'
        ) as f:
            self.resource_block_collection_mockup = json.load(f)

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated
        try:
            resource_block_collection = ResourceBlockCollection(
                self.server_hardware_list,
                self.server_profile_template_list,
                self.drives_list)
        except Exception as e:
            self.fail("Failed to instantiate ResourceBlockCollection class."
                      " Error: {}".format(e))
        self.assertIsInstance(
            resource_block_collection, ResourceBlockCollection)

    def test_serialize(self):
        # Tests the serialize function result against known result
        try:
            resource_block_collection = ResourceBlockCollection(
                self.server_hardware_list,
                self.server_profile_template_list,
                self.drives_list)
        except Exception as e:
            self.fail("Failed to instantiate ResourceBlockCollection class."
                      " Error: {}".format(e))

        try:
            result = json.loads(resource_block_collection.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: {}".format(e))

        self.assertEqualMockup(self.resource_block_collection_mockup, result)

    def test_serialize_empty_result(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'ResourceBlockCollectionEmpty.json'
        ) as f:
            expected_result = json.load(f)

        # Tests the serialize function result against empty list result
        resource_block_collection = ResourceBlockCollection()
        result = json.loads(resource_block_collection.serialize())

        self.assertEqualMockup(expected_result, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json
from unittest import mock

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints import event_service
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest
from oneview_redfish_toolkit import util


@mock.patch.object(config, 'auth_mode_is_conf')
class TestEventService(BaseFlaskTest):
    """Tests for EventService blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestEventService, self).setUpClass()

        self.app.register_blueprint(event_service.event_service)

    @mock.patch('oneview_redfish_toolkit.util.get_delivery_retry_attempts')
    @mock.patch('oneview_redfish_toolkit.util.get_delivery_retry_interval')
    def test_get_event_service_when_is_enabled(self,
                                               get_delivery_retry_interval,
                                               get_delivery_retry_attempts,
                                               auth_mode_is_conf):
        """Tests get EventService when Event Service is enabled"""
        get_delivery_retry_interval.return_value = 30
        get_delivery_retry_attempts.return_value = 3
        auth_mode_is_conf.return_value = True

        response = self.client.get("/redfish/v1/EventService/")

        result = json.loads(response.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/redfish/EventService.json'
        ) as f:
            event_service_mockup = json.loads(f.read())

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(event_service_mockup, result)

    @mock.patch('oneview_redfish_toolkit.util.get_delivery_retry_attempts')
    @mock.patch('oneview_redfish_toolkit.util.get_delivery_retry_interval')
    def test_get_event_service_when_is_disabled(self,
                                                get_delivery_retry_interval,
                                                get_delivery_retry_attempts,
                                                auth_mode_is_conf):
        """Tests get EventService when Event Service is disabled"""
        get_delivery_retry_interval.return_value = 10
        get_delivery_retry_attempts.return_value = 2
        auth_mode_is_conf.return_value = False

        response = self.client.get("/redfish/v1/EventService/")

        result = json.loads(response.data.decode("utf-8"))

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'EventServiceDisabled.json'
        ) as f:
            event_service_mockup = json.loads(f.read())

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(event_service_mockup, result)

    def test_post_submit_event_without_event_type(self, auth_mode_is_conf):
        """Tests EventService SubmitTestEvent action without EventType"""
        auth_mode_is_conf.return_value = True

        response = self.client.post(
            '/redfish/v1/EventService/Actions/EventService.SubmitTestEvent/')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual('application/json', response.mimetype)

    def test_submit_event_with_invalid_event_type(self, auth_mode_is_conf):
        """Tests EventService SubmitTestEvent action with invalid EventType"""
        auth_mode_is_conf.return_value = True

        response = self.client.post(
            '/redfish/v1/EventService/Actions/EventService.SubmitTestEvent/',
            data='{"EventType": "INVALID"}',
            content_type='application/json')

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual('application/json', response.mimetype)

    @mock.patch.object(util, 'dispatch_event')
    def test_submit_event_with_exception(self, dispatch_event_mock,
                                         auth_mode_is_conf):
        """Tests SubmitTestEvent when dispatch_event raises an exception"""
        auth_mode_is_conf.return_value = True
        dispatch_event_mock.side_effect = Exception()

        response = self.client.post(
            '/redfish/v1/EventService/Actions/EventService.SubmitTestEvent/',
            data='{"EventType": "Alert"}',
            content_type='application/json')

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR, response.status_code)

    @mock.patch.object(util, 'dispatch_event')
    def test_submit_event_correct_response(self, _, auth_mode_is_conf):
        """Tests SubmitTestEvent action with OneView SCMB alert"""
        auth_mode_is_conf.return_value = True

        # Loading Alert mockup value
        with open(
            'oneview_redfish_toolkit/mockups/redfish/Alert.json'
        ) as f:
            self.alert_mockup = json.load(f)

        response = self.client.post(
            '/redfish/v1/EventService/Actions''/EventService.SubmitTestEvent/',
            data='{"EventType": "Alert"}',
            content_type='application/json')

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqualMockup(self.alert_mockup, result)
        self.assertEqual(status.HTTP_202_ACCEPTED, response.status_code)
        self.assertEqual('application/json', response.mimetype)

    def test_submit_event_when_is_disabled(self, auth_mode_is_conf):
        """Tests SubmitTestEvent when EventService is disabled"""
        auth_mode_is_conf.return_value = False

        response = self.client.post(
            '/redfish/v1/EventService/Actions''/EventService.SubmitTestEvent/',
            data='{"EventType": "Alert"}',
            content_type='application/json')

        result = response.data.decode("utf-8")

        self.assertIn('EventService is not enabled.', str(result))
        self.assertEqual(status.HTTP_404_NOT_FOUND,
                         response.status_code)
        self.assertEqual('application/json', response.mimetype)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.odata import Odata
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestOdata(BaseTest):
    """Tests for Odata class"""

    def test_class_instantiation(self):
        """Tests class instantiation and validation"""

        try:
            odata = Odata()
        except Exception as e:
            self.fail("Failed to instantiate Odata. Error: ".format(e))
        self.assertIsInstance(odata, Odata)

    def test_serialize(self):
        """Tests the serialize function result against known result"""

        odata = Odata()
        result = json.loads(odata.serialize())

        with open(
            'oneview_redfish_toolkit/mockups/redfish/Odata.json'
        ) as f:
            odata_mockup = json.load(f)
        self.assertEqualMockup(odata_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import os
import shutil
from unittest import mock

# 3rd party libs
from hpOneView.exceptions import HPOneViewException

# Own libs
from oneview_redfish_toolkit.api import scmb
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit.tests.base_test import BaseTest
from oneview_redfish_toolkit import util


class TestSCMB(BaseTest):
    """Tests for SCMB module"""

    @mock.patch.object(scmb, '_is_cert_working_with_scmb')
    @mock.patch.object(scmb, 'config')
    @mock.patch('os.path.isfile')
    def test_check_cert_exist(self, isfile, config_mock,
                              _is_cert_working_with_scmb):
        config_mock.get_config.return_value = {
            'ssl': {
                'SSLCertFile': ''
            }
        }

        # Files exist
        isfile.return_value = True
        _is_cert_working_with_scmb.return_value = True
        self.assertTrue(scmb._has_valid_certificates())

        # Certs files don't exist
        isfile.return_value = False
        _is_cert_working_with_scmb.return_value = False
        self.assertFalse(scmb._has_valid_certificates())
        self.assertFalse(scmb._has_valid_certificates())

    @mock.patch.object(scmb, 'config')
    def test_paths_generated_for_scmb_files(self, config_mock):
        config_mock.get_config.return_value = {
            'ssl': {
                'SSLCertFile': '/dir/cert_file.crt'
            }
        }

        self.assertEqual('/dir/scmb/oneview_ca.pem', scmb._oneview_ca_path())
        self.assertEqual('/dir/scmb/oneview_scmb.pem', scmb._scmb_cert_path())
        self.assertEqual('/dir/scmb/oneview_scmb.key', scmb._scmb_key_path())

    @mock.patch.object(scmb, 'config')
    @mock.patch.object(scmb, 'logging')
    def test_check_cert_exist_when_config_key_is_missing(self,
                                                         logging_mock,
                                                         config_mock):
        config_mock.get_config.return_value = {
            'ssl': {}
        }

        with self.assertRaises(KeyError) as error:
            scmb._has_valid_certificates()

        logging_mock.error.assert_called_with(
            'Invalid configuration for ssl cert. '
            'Verify the [ssl] section in config file')

        self.assertEqual("'SSLCertFile'", str(error.exception))

    @mock.patch.object(scmb, '_is_cert_working_with_scmb')
    @mock.patch.object(scmb, 'config')
    @mock.patch.object(scmb, 'get_oneview_client')
    def test_get_cert_already_exists(self, get_oneview_client, config_mock,
                                     _is_cert_working_with_scmb):
        config_mock.get_config.return_value = {
            'ssl': {
                'SSLCertFile': 'cert_file.crt'
            }
        }

        os.makedirs(name='scmb', exist_ok=True)
        self.addCleanup(shutil.rmtree, 'scmb')

        # Certs Generated with success
        oneview_client = mock.MagicMock()
        oneview_client.certificate_authority.get.return_value = "CA CERT"
        oneview_client.certificate_rabbitmq.generate.return_value = True
        oneview_client.certificate_rabbitmq.get_key_pair.return_value = {
            'base64SSLCertData': 'Client CERT',
            'base64SSLKeyData': 'Client Key'}
        get_oneview_client.return_value = oneview_client
        _is_cert_working_with_scmb.return_value = True

        # Certs already exist
        e = HPOneViewException({
            'errorCode': 'RABBITMQ_CLIENTCERT_CONFLICT',
            'message': 'certs already exist',
        })
        oneview_client.certificate_rabbitmq.generate.side_effect = e
        scmb.get_cert()
        self.assertTrue(scmb._has_valid_certificates())

    @mock.patch('pika.BlockingConnection')
    @mock.patch('pika.ConnectionParameters')
    @mock.patch('pika.credentials.ExternalCredentials')
    def test_scmb_connect(self, blk_conn, conn_params, ext_cred):
        blk_conn.return_value = {}
        conn_params.return_value = {}
        ext_cred.return_value = {}
        self.assertEqual(scmb.scmb_connect(), {})

    @mock.patch.object(util, 'dispatch_event')
    def test_consume_message(self, dispatch_mock):
        with open(
            'oneview_redfish_toolkit/mockups/oneview/Alert.json'
        ) as f:
            event_mockup = f.read().encode('UTF-8')

        scmb.consume_message(None, None, None, event_mockup)

        self.assertTrue(dispatch_mock.called)

    @mock.patch.object(scmb, 'config')
    @mock.patch.object(scmb, '_is_cert_working_with_scmb')
    @mock.patch.object(scmb, 'get_oneview_client')
    def test_generate_new_cert_for_oneview(self, get_oneview_client,
                                           _is_cert_working_with_scmb,
                                           config_mock):
        config_mock.get_config.return_value = {
            'ssl': {
                'SSLCertFile': 'cert_file.crt'
            }
        }

        cert_key_pair = {
            'base64SSLCertData': 'Client CERT',
            'base64SSLKeyData': 'Client Key'
        }

        # Certs Generated with success
        oneview_client = mock.MagicMock()
        e = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'Resource not found.',
        })

        oneview_client.certificate_authority.get.return_value = "CA CERT"
        oneview_client.certificate_rabbitmq.generate.return_value = True
        oneview_client.certificate_rabbitmq.get_key_pair.side_effect = \
            [e, cert_key_pair]
        get_oneview_client.return_value = oneview_client
        _is_cert_working_with_scmb.return_value = True
        scmb.get_cert()
        self.assertTrue(scmb._has_valid_certificates())

    @mock.patch.object(scmb, 'config')
    @mock.patch.object(scmb, 'get_oneview_client')
    def test_get_oneview_cert_unexpected_error(self, get_oneview_client,
                                               config_mock):
        config_mock.get_config.return_value = {
            'ssl': {
                'SSLCertFile': 'cert_file.crt'
            }
        }

        # Certs Generated with success
        oneview_client = mock.MagicMock()
        e = HPOneViewException({
            'errorCode': 'UNEXPECTED ERROR',
            'message': 'Unexpected error.',
        })
        hp_ov_exception_msg = "Unexpected error."

        oneview_client.certificate_authority.get.return_value = "CA CERT"
        oneview_client.certificate_rabbitmq.generate.return_value = True
        oneview_client.certificate_rabbitmq.get_key_pair.side_effect = e
        get_oneview_client.return_value = oneview_client

        with self.assertRaises(HPOneViewException) as hp_exception:
            scmb.get_cert()

        test_exception = hp_exception.exception
        self.assertEqual(hp_ov_exception_msg, test_exception.msg)
        self.assertEqual(e.oneview_response, test_exception.oneview_response)

    @mock.patch('pika.channel.Channel')
    @mock.patch('pika.BlockingConnection')
    @mock.patch('pika.ConnectionParameters')
    @mock.patch.object(scmb, 'config')
    @mock.patch.object(scmb, 'get_oneview_client')
    def test_init_event_service_with_valid_certificate(self,
                                                       get_oneview_client,
                                                       config_mock, conn_param,
                                                       block_conn, channel):
        config_mock.get_config.return_value = {
            'ssl': {
                'SSLCertFile': 'cert_file.crt'
            },
        }

        os.makedirs(name='scmb', exist_ok=True)
        self.addCleanup(shutil.rmtree, 'scmb')

        oneview_client = mock.MagicMock()
        oneview_client.certificate_authority.get.return_value = "CA CERT"
        oneview_client.certificate_rabbitmq.generate.return_value = True
        get_oneview_client.return_value = oneview_client
        oneview_client.certificate_rabbitmq.get_key_pair.return_value = {
            'base64SSLCertData': 'Client CERT',
            'base64SSLKeyData': 'Client Key'}

        pika_mock = mock.MagicMock()
        pika_mock.channel.Channel = {}
        block_conn.return_value = pika_mock
        conn_param.return_value = {}
        channel.return_value = {}

        scmb.init_event_service()

        self.assertTrue(scmb._has_valid_certificates())
        self.assertTrue(scmb._is_cert_working_with_scmb())

    @mock.patch.object(config, 'config')
    def test_init_event_service_on_session_mode(self, config_mock):
        config_mock.get_authentication_mode.return_value = 'session'
        scmb.init_event_service()

        self.assertFalse(scmb._has_valid_certificates())
        self.assertFalse(scmb._is_cert_working_with_scmb())

    @mock.patch('pika.channel.Channel')
    @mock.patch('pika.BlockingConnection')
    @mock.patch('pika.ConnectionParameters')
    @mock.patch.object(scmb, '_has_scmb_certificates_path')
    @mock.patch.object(scmb, 'get_oneview_client')
    @mock.patch.object(scmb, 'config')
    def test_init_event_service_with_certs_already_generated(self, config_mock,
                                                             get_ov_client,
                                                             _has_certs_path,
                                                             conn_param,
                                                             block_conn,
                                                             channel):
        config_mock.get_config.return_value = {
            'ssl': {
                'SSLCertFile': '/dir/cert_file.crt'
            },
        }

        pika_mock = mock.MagicMock()
        pika_mock.channel.Channel = {}
        block_conn.return_value = pika_mock
        conn_param.return_value = {}
        channel.return_value = {}
        oneview_client = mock.MagicMock()
        get_ov_client.return_value = oneview_client
        _has_certs_path.return_value = True

        scmb.init_event_service()

        self.assertTrue(scmb._has_valid_certificates())



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask_api import status

# own libs
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishResourceNotFoundError
from oneview_redfish_toolkit.api.network_interface import NetworkInterface
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

network_interface = Blueprint("network_interface", __name__)


@network_interface.route(
    ComputerSystem.BASE_URI +
    "/<server_profile_uuid>/NetworkInterfaces/<device_id>",
    methods=["GET"])
def get_network_interface(server_profile_uuid, device_id):
    """Get the Redfish NetworkInterface for a given UUID and device_id.

        Return NetworkInterface Redfish JSON for a given hardware UUID
        and device_id.

        Parameters:
            server_profile_uuid: the UUID of the server profile
            device_id: The id of the network device

        Returns:
            JSON: Redfish json with NetworkInterface

        Exceptions:
            When server profile or hardware is not found calls abort(404)
            When other errors occur calls abort(500)

    """
    try:
        device_id_validation = int(device_id)

        profile = g.oneview_client.server_profiles.get(server_profile_uuid)
        server_hardware = g.oneview_client.server_hardware\
            .get(profile["serverHardwareUri"])

        if (device_id_validation - 1) < 0 or (device_id_validation - 1) >= \
                len(server_hardware["portMap"]["deviceSlots"]):
            raise OneViewRedfishResourceNotFoundError(
                device_id, "Network interface")

        ni = NetworkInterface(device_id, profile, server_hardware)

        return ResponseBuilder.success(ni)

    except ValueError:
        # Failed to convert device_id to int
        logging.exception(
            "Failed to convert device id {} to integer.".format(device_id))
        abort(status.HTTP_404_NOT_FOUND, "Network interface not found")
    except OneViewRedfishResourceNotFoundError as e:
        logging.exception(e.msg)
        abort(status.HTTP_404_NOT_FOUND, e.msg)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import copy
import json

# 3rd party libs
from unittest.mock import call

from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import storage
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestStorage(BaseFlaskTest):
    """Tests for Storage blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestStorage, self).setUpClass()

        self.app.register_blueprint(storage.storage)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerProfile.json'
        ) as f:
            self.server_profile = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardwareTypes.json'
        ) as f:
            self.server_hardware_type = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'SASLogicalJBODListForStorage.json'
        ) as f:
            self.logical_jbods = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/Storage.json'
        ) as f:
            self.storage_mockup = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/redfish/Drive.json'
        ) as f:
            self.drive_mockup = json.load(f)

        self.not_found_error = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'some message not found',
        })

    def test_get_storage(self):
        """Tests Storage"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.server_hardware_types.get.return_value \
            = self.server_hardware_type
        self.oneview_client.\
            sas_logical_jbods.get.side_effect = self.logical_jbods

        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1"
        )

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.storage_mockup, result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.server_hardware_types.get.assert_called_with(
            self.server_hardware_type["uri"])
        self.oneview_client.sas_logical_jbods.get.assert_has_calls(
            [
                call(self.logical_jbods[0]["uri"]),
                call(self.logical_jbods[1]["uri"])
            ]
        )

    def test_get_storage_when_profile_not_found(self):
        """Tests when server profile not found"""

        self.oneview_client.\
            server_profiles.get.side_effect = self.not_found_error

        # Get Storage
        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_storage_when_get_profile_raises_any_exception(self):
        """Tests when the searching of server profile raises an error"""

        self.oneview_client.server_profiles.get.side_effect = Exception

        # Get Storage
        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    def test_get_storage_when_hardware_type_not_found(self):
        """Tests when server hardware type not found"""

        self.oneview_client.server_hardware_types.get.side_effect = \
            self.not_found_error

        # Get Storage
        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_storage_when_hardware_type_raises_any_exception(self):
        """Tests when the searching of server hardware type raises an error"""

        self.oneview_client.server_hardware_types.get.side_effect = Exception

        # Get Storage
        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_drive(self):
        """Tests get a valid Drive"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.\
            sas_logical_jbods.get.side_effect = self.logical_jbods

        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1/Drives/4"
        )

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.drive_mockup, result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.sas_logical_jbods.get.assert_has_calls(
            [
                call(self.logical_jbods[0]["uri"]),
                call(self.logical_jbods[1]["uri"])
            ]
        )

    def test_get_drive_when_profile_not_found(self):
        """Tests when server profile not found"""

        self.oneview_client.server_profiles.get.side_effect = \
            self.not_found_error

        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1/Drives/4"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.sas_logical_jbods.get.assert_not_called()

    def test_get_drive_when_profile_raises_any_exception(self):
        """Tests when the searching of server profile raises any error"""

        self.oneview_client.server_profiles.get.side_effect = Exception

        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1/Drives/4"
        )

        self.assertEqual(status.HTTP_500_INTERNAL_SERVER_ERROR,
                         response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.sas_logical_jbods.get.assert_not_called()

    def test_get_drive_when_sas_logical_jbod_not_found(self):
        """Tests when sas logical jbod not found"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.sas_logical_jbods.get.side_effect = \
            self.not_found_error

        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1/Drives/4"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.sas_logical_jbods.get.assert_called_with(
            self.logical_jbods[0]["uri"]
        )

    def test_get_drive_when_drive_not_found(self):
        """Tests when drive id can't be found"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.\
            sas_logical_jbods.get.side_effect = self.logical_jbods

        # we have the 4 drives, so id '5' is invalid
        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1/Drives/5"
        )

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertIn("Drive 5 not found", str(response.data))
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.sas_logical_jbods.get.assert_has_calls(
            [
                call(self.logical_jbods[0]["uri"]),
                call(self.logical_jbods[1]["uri"])
            ]
        )

    def test_get_drive_when_drive_id_is_invalid(self):
        """Tests when drive id is not a number"""

        self.oneview_client.\
            server_profiles.get.return_value = self.server_profile
        self.oneview_client.\
            sas_logical_jbods.get.side_effect = self.logical_jbods

        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1/Drives/abc"
        )

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertIn("Drive id should be a integer", str(response.data))
        self.oneview_client.server_profiles.get.assert_not_called()
        self.oneview_client.sas_logical_jbods.get.assert_not_called()

    def test_composed_system_without_drives(self):
        """Tests Storage when it does not have drives"""
        server_profile = copy.deepcopy(self.server_profile)
        server_profile["localStorage"]["sasLogicalJBODs"] = []
        storage_mockup_without_drives = copy.deepcopy(self.storage_mockup)
        storage_mockup_without_drives["Drives"] = []
        storage_mockup_without_drives["Drives@odata.count"] = 0
        self.oneview_client.server_profiles.get.return_value = server_profile
        self.oneview_client.server_hardware_types.get.return_value \
            = self.server_hardware_type

        response = self.client.get(
            "/redfish/v1/Systems/"
            "b425802b-a6a5-4941-8885-aab68dfa2ee2/Storage/1"
        )

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(storage_mockup_without_drives, result)
        self.oneview_client.server_profiles.get.assert_called_with(
            self.server_profile["uuid"])
        self.oneview_client.server_hardware_types.get.assert_called_with(
            self.server_hardware_type["uri"])
        self.oneview_client.sas_logical_jbods.get.assert_not_called()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json

# 3rd party libs
from flask_api import status
from hpOneView.exceptions import HPOneViewException
from unittest import mock

# Module libs
from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator
from oneview_redfish_toolkit.blueprints import network_port_collection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestNetworkPortCollection(BaseFlaskTest):
    """Tests for NetworkPortCollection blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestNetworkPortCollection, self).setUpClass()

        self.app.register_blueprint(
            network_port_collection.network_port_collection)

        # Loading server_hardware mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

    def test_get_network_port_collection(self):
        """Tests NetworkInterfaceCollection"""

        # Loading NetworkPortCollection mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'NetworkPortCollection.json'
        ) as f:
            network_port_collection_mockup = json.load(f)

        # Create mock response
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware

        # Get NetworkPortCollection
        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752/"
            "NetworkAdapters/1/NetworkPorts/"
        )

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(network_port_collection_mockup, result)

    @mock.patch.object(RedfishJsonValidator, "get_resource_by_id")
    def test_get_network_port_collection_empty(self, get_resource_by_id):
        """Tests NetworkPortCollection empty server hardware"""

        # Loading NetworkPortCollectionEmpty mockup result
        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'NetworkPortCollectionEmpty.json'
        ) as f:
            network_port_collection_empty = json.load(f)

        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware

        # Get NetworkPortCollection
        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752/"
            "NetworkAdapters/1/NetworkPorts/"
        )

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(network_port_collection_empty, result)

    def test_get_network_port_collection_sh_exception(self):
        """Tests NetworkPortCollection with exception"""

        e = HPOneViewException({
            'errorCode': 'ANOTHER_ERROR',
            'message': 'server-hardware-types error',
        })
        self.oneview_client.server_hardware.get.side_effect = e

        # Get NetworkPortCollection
        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752/"
            "NetworkAdapters/1/NetworkPorts/"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)

    def test_get_network_port_collection_invalid_device_id(self):
        """Tests NetworkPortCollection with invalid device id"""

        self.oneview_client.server_hardware.get.side_effect = \
            Exception("Invalid id for device")

        # Get invalid NetworkPortCollection
        response = self.client.get(
            "/redfish/v1/Chassis/30303437-3034-4D32-3230-313133364752/"
            "NetworkAdapters/100/NetworkPorts/"
        )

        self.assertEqual(
            status.HTTP_500_INTERNAL_SERVER_ERROR,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json

# 3rd party libs
from flask_api import status

# Module libs
from oneview_redfish_toolkit.blueprints import resource_block_collection
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestResourceBlockCollection(BaseFlaskTest):
    """Tests for ResourceBlockCollection blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestResourceBlockCollection, self).setUpClass()

        self.app.register_blueprint(
            resource_block_collection.resource_block_collection)

    def test_get_resource_block_collection(self):
        """Tests ResourceBlockCollection"""

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardwareList.json'
        ) as f:
            server_hardware_list = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'ServerProfileTemplates.json'
        ) as f:
            server_profile_template_list = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Drives.json'
        ) as f:
            drives_list = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'ResourceBlockCollection.json'
        ) as f:
            resource_block_collection_mockup = json.load(f)

        self.oneview_client.server_hardware.get_all.return_value = \
            server_hardware_list

        self.oneview_client.server_profile_templates.get_all.return_value = \
            server_profile_template_list

        self.oneview_client.index_resources.get_all.return_value = \
            drives_list

        # Get ResourceBlockCollection
        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks/")

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(resource_block_collection_mockup, result)

    def test_get_resource_block_collection_empty(self):
        """Tests ResourceBlockCollection with an empty list"""

        self.oneview_client.server_hardware.get_all.return_value = []
        self.oneview_client.server_profile_templates.get_all.return_value = []
        self.oneview_client.index_resources.get_all.return_value = []

        # Get ResourceBlockCollection
        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks/")

        # Gets json from response
        result = json.loads(response.data.decode("utf-8"))

        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'ResourceBlockCollectionEmpty.json'
        ) as f:
            expected_result = json.load(f)

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_result, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for query_ov_client_by_resource and
    search_resource_multiple_ov function from multiple_oneview.py
"""
import collections
import configparser
import json
import unittest

from unittest import mock
from unittest.mock import call

from hpOneView.exceptions import HPOneViewException

from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import handler_multiple_oneview
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit import single_oneview_context


@mock.patch.object(config, 'get_config')
@mock.patch.object(client_session, 'request')
@mock.patch.object(connection, 'OneViewClient')
@mock.patch.object(client_session, 'get_oneview_client')
@mock.patch.object(single_oneview_context, 'g')
class TestMultipleOneView(unittest.TestCase):
    """Test class for multiple_oneview"""

    @classmethod
    def setUpClass(self):
        super(TestMultipleOneView, self).setUpClass()

        # Loading server_profile mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerProfile.json'
        ) as f:
            self.server_profile = json.load(f)

        self.sp_uuid = self.server_profile['uri'].split('/')[-1]

        self.not_found_server_profile = HPOneViewException({
            'errorCode': 'ProfileNotFoundException',
            'message': 'The requested profile cannot be retrieved',
        })

        self.not_found_error = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'The requested resource cannot be retrieved',
        })

        # Creating mocked redfish->Oneview tokens
        self.redfish_token = "redfish_tk1"
        self.ov_tokens = collections.OrderedDict()
        self.ov_tokens["10.0.0.1"] = "ov_tk1"
        self.ov_tokens["10.0.0.2"] = "ov_tk2"
        self.ov_tokens["10.0.0.3"] = "ov_tk3"

    def setUp(self):
        # Initializing caches
        multiple_oneview.init_map_resources()
        client_session.init_map_clients()
        category_resource.init_map_category_resources()

        self.config_obj = configparser.ConfigParser()
        self.config_obj.add_section('oneview_config')
        self.config_obj.add_section('redfish')

    def test_search_in_all_ov_found_on_second(self, req_context,
                                              get_oneview_client,
                                              oneview_client_mockup,
                                              request,
                                              get_config):
        # Mocking configuration read from config file
        self.config_obj.set('oneview_config', 'ip', '10.0.0.1, 10.0.0.2')
        self.config_obj.set('redfish', 'authentication_mode', 'session')

        # Mocking globals['config'] of config file
        get_config.return_value = self.config_obj

        # Mocking redfish->Oneview tokens
        client_session._set_new_client_by_token(self.redfish_token,
                                                self.ov_tokens)

        # Mocking redfish token on request property
        request.headers.get.return_value = self.redfish_token

        # Mocking OneView client call returning resource for the second
        # one, ip: 10.0.0.2
        oneview_client_mockup.server_profiles.get.side_effect = [
            self.not_found_server_profile,
            self.server_profile,
        ]

        # Mocking connection.get_oneview_client() to return mocked
        # OneView Client
        get_oneview_client.return_value = oneview_client_mockup

        # Create new handler for multiple OneView support
        handler_multiple_ov = \
            handler_multiple_oneview.MultipleOneViewResource()

        # Query resource
        handler_multiple_ov.server_profiles.get(self.sp_uuid)

        # Check if resource was queried on two of three OneViews
        oneview_client_mockup.server_profiles.get.assert_has_calls(
            [call(self.sp_uuid),
             call(self.sp_uuid)]
        )

        # Check if resource was queried on first and the second OneViews
        get_oneview_client.assert_has_calls(
            [call("10.0.0.1"),
             call("10.0.0.2")]
        )

    def test_search_in_all_ov_when_auth_mode_is_conf(self,
                                                     req_context,
                                                     get_oneview_client,
                                                     oneview_client_mockup,
                                                     request,
                                                     get_config):
        # Mocking configuration read from config file
        self.config_obj.set('oneview_config', 'ip', '10.0.0.1, oneview.com')
        self.config_obj.set('redfish', 'authentication_mode', 'conf')

        # Mocking globals['config'] of config file
        get_config.return_value = self.config_obj

        # Mocking redfish token on request to be None due the conf mode
        request.headers.get.return_value = None

        # Mocking OneView client call returning resource for the second
        # one, ip: 10.0.0.2
        oneview_client_mockup.server_profiles.get.side_effect = [
            self.not_found_server_profile,
            self.server_profile,
        ]

        # Mocking connection.get_oneview_client() to return mocked
        # OneView Client
        get_oneview_client.return_value = oneview_client_mockup

        # Create new handler for multiple OneView support
        handler_multiple_ov = \
            handler_multiple_oneview.MultipleOneViewResource()

        # Query resource
        handler_multiple_ov.server_profiles.get(self.sp_uuid)

        # Check if resource was queried on two of three OneViews
        oneview_client_mockup.server_profiles.get.assert_has_calls(
            [call(self.sp_uuid),
             call(self.sp_uuid)]
        )

        get_oneview_client.assert_has_calls(
            [call("10.0.0.1"),
             call("oneview.com")]
        )

    def test_search_mapped_after_search_in_all(self, req_context,
                                               get_oneview_client,
                                               oneview_client_mockup,
                                               request,
                                               get_config):
        # Mocking configuration read from config file
        self.config_obj.set('oneview_config', 'ip',
                            '10.0.0.1, 10.0.0.2, 10.0.0.3')
        self.config_obj.set('redfish', 'authentication_mode', 'session')

        # Mocking globals['config'] of config file
        get_config.return_value = self.config_obj

        # Mocking redfish->Oneview tokens
        client_session._set_new_client_by_token(self.redfish_token,
                                                self.ov_tokens)

        # Mocking redfish token on request property
        request.headers.get.return_value = self.redfish_token

        # Mocking OneView client call returning resource just for the last
        # one, ip: 10.0.0.3
        oneview_client_mockup.server_profiles.get.side_effect = [
            self.not_found_server_profile,
            self.not_found_server_profile,
            self.server_profile,
        ]

        # Mocking connection.get_oneview_client() to return mocked
        # OneView Client
        get_oneview_client.return_value = oneview_client_mockup

        # Create new handler for multiple OneView support
        handler_multiple_ov = \
            handler_multiple_oneview.MultipleOneViewResource()

        # Query resource
        handler_multiple_ov.server_profiles.get(self.sp_uuid)

        # Check if resource was queried for all OneViews before found on
        # last one
        oneview_client_mockup.server_profiles.get.assert_has_calls(
            [call(self.sp_uuid),
             call(self.sp_uuid),
             call(self.sp_uuid)]
        )

        # Check if resource was queried on each one for all OneViews
        get_oneview_client.assert_has_calls(
            [call("10.0.0.1"),
             call("10.0.0.2"),
             call("10.0.0.3")]
        )

        # Mocking OneView client call returning resource
        oneview_client_mockup.server_profiles.get.side_effect = [
            self.server_profile,
        ]

        # Query resource again, now already mapped on cache
        handler_multiple_ov.server_profiles.get(self.sp_uuid)

        # Check if resource was queried on just one OneView, the mapped one
        # for the resource
        oneview_client_mockup.server_profiles.get.assert_has_calls(
            [call(self.sp_uuid)]
        )

        # Check if resource was queried on just one OneViews that was mapped
        # for the resource
        get_oneview_client.assert_has_calls(
            [call("10.0.0.3")]
        )

        # Get the OneView IP mapped for the resource
        mapped_ov_ip = multiple_oneview.get_ov_ip_by_resource(self.sp_uuid)

        # Check OneView IP matchs the mapped one
        self.assertEqual(mapped_ov_ip, "10.0.0.3")

    def test_searching_again_in_other_ov_when_resource_cached_is_not_found(
            self,
            _,
            get_oneview_client,
            oneview_client_mockup,
            request,
            get_config):
        self.config_obj.set('oneview_config', 'ip',
                            '10.0.0.1, 10.0.0.2, 10.0.0.3')
        self.config_obj.set('redfish', 'authentication_mode', 'session')

        get_config.return_value = self.config_obj

        client_session._set_new_client_by_token(self.redfish_token,
                                                self.ov_tokens)

        # Mocking redfish token on request property
        request.headers.get.return_value = self.redfish_token

        with open(
                'oneview_redfish_toolkit/mockups/oneview/Enclosure.json'
        ) as f:
            enclosure = json.load(f)

        oneview_client_mockup.enclosures.get.side_effect = [
            self.not_found_error,  # 10.0.0.1
            enclosure,  # 10.0.0.2 found the resource here
            self.not_found_error,  # 10.0.0.2
            self.not_found_error,  # 10.0.0.1
            enclosure,  # 10.0.0.3 found the resource here (2th time)
            self.not_found_error,  # 10.0.0.3
            self.not_found_error,  # 10.0.0.1
            self.not_found_error  # 10.0.0.2
        ]

        get_oneview_client.return_value = oneview_client_mockup
        handler_multiple_ov = \
            handler_multiple_oneview.MultipleOneViewResource()

        handler_multiple_ov.enclosures.get('UUID_1')
        mapped_ov_ip = multiple_oneview.get_ov_ip_by_resource('UUID_1')

        # Check OneView IP matchs the mapped one
        self.assertEqual(mapped_ov_ip, '10.0.0.2')
        get_oneview_client.assert_has_calls(
            [call("10.0.0.1"),
             call("10.0.0.2")]
        )

        handler_multiple_ov.enclosures.get('UUID_1')
        mapped_ov_ip = multiple_oneview.get_ov_ip_by_resource('UUID_1')

        self.assertEqual(mapped_ov_ip, '10.0.0.3')
        get_oneview_client.assert_has_calls(
            [call("10.0.0.2"),
             call("10.0.0.1"),
             call("10.0.0.3")]
        )

        with self.assertRaises(HPOneViewException):
            handler_multiple_ov.enclosures.get('UUID_1')

        get_oneview_client.assert_has_calls(
            [call("10.0.0.3"),
             call("10.0.0.1"),
             call("10.0.0.2")]
        )

    def test_search_resource_not_found(self, _, get_oneview_client,
                                       oneview_client_mockup,
                                       request,
                                       get_config):
        self.config_obj.set('oneview_config', 'ip',
                            '10.0.0.1, 10.0.0.2, 10.0.0.3')
        self.config_obj.set('redfish', 'authentication_mode', 'session')

        get_config.return_value = self.config_obj

        client_session._set_new_client_by_token(self.redfish_token,
                                                self.ov_tokens)

        # Mocking redfish token on request property
        request.headers.get.return_value = self.redfish_token

        oneview_client_mockup.enclosures.get.side_effect = [
            self.not_found_error,  # 10.0.0.1
            self.not_found_error,  # 10.0.0.2
            self.not_found_error,  # 10.0.0.3
        ]

        get_oneview_client.return_value = oneview_client_mockup
        handler_multiple_ov = \
            handler_multiple_oneview.MultipleOneViewResource()

        with self.assertRaises(HPOneViewException):
            handler_multiple_ov.enclosures.get('UUID_1')

        get_oneview_client.assert_has_calls(
            [call("10.0.0.1"),
             call("10.0.0.2"),
             call("10.0.0.3")]
        )



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from collections import OrderedDict
import copy
import json
from unittest import mock

# 3rd party libs
from flask_api import status

# Module libs
import oneview_redfish_toolkit.api.status_mapping as status_mapping
from oneview_redfish_toolkit.blueprints import manager
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestManager(BaseFlaskTest):
    """Tests for Managers blueprint

        Tests:
            - manager
                - know value
                - not found error
    """

    @classmethod
    def setUpClass(self):
        super(TestManager, self).setUpClass()

        self.app.register_blueprint(manager.manager)

        # Loading ApplianceNodeInfo mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ApplianceNodeInfo.json'
        ) as f:
            self.appliance_info = json.load(f)

        # Loading ApplianceState mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ApplianceState.json'
        ) as f:
            self.appliance_state = json.load(f)

        # Loading ApplianceHealthStatus mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ApplianceHealthStatus.json'
        ) as f:
            self.appliance_health_status = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'Manager.json'
        ) as f:
            self.manager_mockup = json.load(f)

        self.map_appliance = OrderedDict({
            "10.0.0.1": self.appliance_info["uuid"]
        })

    @mock.patch.object(client_session, 'get_oneview_client')
    @mock.patch.object(multiple_oneview, 'execute_query_ov_client')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_manager(self, get_map_appliances, execute_query_ov_client,
                         get_oneview_client):
        """"Tests get Manager"""

        get_map_appliances.return_value = self.map_appliance
        get_oneview_client.return_value = self.oneview_client
        execute_query_ov_client.side_effect = [
            self.appliance_info, self.appliance_state,
            self.appliance_health_status
        ]

        # Get Manager
        response = self.client.get(
            "/redfish/v1/Managers/b08eb206-a904-46cf-9172-dcdff2fa9639"
        )

        result = json.loads(response.data.decode("utf-8"))

        # Tests response
        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.manager_mockup, result)

    @mock.patch.object(client_session, 'get_oneview_client')
    @mock.patch.object(multiple_oneview, 'execute_query_ov_client')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_all_manager_states(self, get_map_appliances,
                                execute_query_ov_client,
                                get_oneview_client):
        """"Tests get Manager with all possible States"""

        appliance_state = copy.deepcopy(self.appliance_info)
        manager_mockup = copy.deepcopy(self.manager_mockup)

        for oneview_state, redfish_state in status_mapping.\
                APPLIANCE_STATE_TO_REDFISH_STATE.items():

            appliance_state["state"] = oneview_state
            manager_mockup["Status"]["State"] = redfish_state

            get_map_appliances.return_value = self.map_appliance
            get_oneview_client.return_value = self.oneview_client
            execute_query_ov_client.side_effect = [
                self.appliance_info, appliance_state,
                self.appliance_health_status
            ]

            # Get Manager
            response = self.client.get(
                "/redfish/v1/Managers/b08eb206-a904-46cf-9172-dcdff2fa9639"
            )

            result = json.loads(response.data.decode("utf-8"))

            # Tests response
            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqualMockup(manager_mockup, result)

    @mock.patch.object(client_session, 'get_oneview_client')
    @mock.patch.object(multiple_oneview, 'execute_query_ov_client')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_all_manager_health_status(self, get_map_appliances,
                                       execute_query_ov_client,
                                       get_oneview_client):
        """Tests get Manager with all possible Health Status"""

        appliance_health_status = \
            copy.deepcopy(self.appliance_health_status)
        manager_mockup = copy.deepcopy(self.manager_mockup)

        for oneview_health_status, redfish_health_status in \
                status_mapping.MANAGER_HEALTH_STATE.items():

            appliance_health_status["members"][0]["severity"] = \
                oneview_health_status
            manager_mockup["Status"]["Health"] = redfish_health_status

            get_map_appliances.return_value = self.map_appliance
            get_oneview_client.return_value = self.oneview_client
            execute_query_ov_client.side_effect = [
                self.appliance_info, self.appliance_state,
                appliance_health_status
            ]

            # Get Manager
            response = self.client.get(
                "/redfish/v1/Managers/b08eb206-a904-46cf-9172-dcdff2fa9639"
            )

            result = json.loads(response.data.decode("utf-8"))

            # Tests response
            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqualMockup(manager_mockup, result)

    @mock.patch.object(client_session, 'get_oneview_client')
    @mock.patch.object(multiple_oneview, 'execute_query_ov_client')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_manager_not_found(self, get_map_appliances,
                               execute_query_ov_client,
                               get_oneview_client):
        """Tests Manager with an unexpected error"""

        get_map_appliances.return_value = self.map_appliance
        get_oneview_client.return_value = self.oneview_client
        execute_query_ov_client.side_effect = [
            self.appliance_info, self.appliance_state,
            self.appliance_health_status
        ]

        response = self.client.get(
            "/redfish/v1/Managers/b08eb206-a904-46cf-9172-dcdff2fa9657"
        )

        self.assertEqual(
            status.HTTP_404_NOT_FOUND,
            response.status_code)
        self.assertEqual("application/json", response.mimetype)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for store_schema and load_registry function from util.py
"""
import collections
from collections import OrderedDict
import unittest
from unittest import mock
from unittest.mock import call

from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import multiple_oneview


class TestAuthentication(unittest.TestCase):
    """Test class for authentication"""

    def setUp(self):
        client_session.init_map_clients()
        multiple_oneview.init_map_appliances()

    @mock.patch.object(client_session, 'uuid')
    @mock.patch('oneview_redfish_toolkit.connection.OneViewClient')
    @mock.patch.object(config, 'get_oneview_multiple_ips')
    @mock.patch.object(config, 'get_authentication_mode')
    def test_map_token_redfish_for_multiple_ov(self, get_authentication_mode,
                                               get_oneview_multiple_ips,
                                               oneview_client_mockup,
                                               uuid_mock):
        get_authentication_mode.return_value = 'session'
        mocked_rf_token = "abc"
        session_id = '123456'
        list_ips = ['10.0.0.1', '10.0.0.2', '10.0.0.3']
        conn_1 = mock.MagicMock()

        connection_list = [conn_1, mock.MagicMock(), mock.MagicMock()]

        connections_ov = collections.OrderedDict({
            'client_ov_by_ip': {
                list_ips[0]: connection_list[0],
                list_ips[1]: connection_list[1],
                list_ips[2]: connection_list[2]
            },
            'session_id': session_id
        })

        uuid_mock.uuid4.return_value = session_id

        iter_conns_ov = iter(connection_list)

        client_session.init_map_clients()

        get_oneview_multiple_ips.return_value = list_ips

        def function_returning_token(ov_config):
            return next(iter_conns_ov)

        oneview_client_mockup.side_effect = function_returning_token
        conn_1.connection.get_session_id.return_value = mocked_rf_token

        # Check if redfish token return is one of the OneView's token
        rf_token, _ = client_session.login('user', 'password')
        oneview_client_mockup.assert_any_call(
            {
                'ip': '10.0.0.1',
                'credentials': {'userName': 'user', 'password': 'password'},
                'api_version': 600
            }
        )
        self.assertEqual(rf_token, mocked_rf_token)

        # Check if cached connection map has the Redfish token return on login
        map_clients = client_session._get_map_clients()
        self.assertTrue(rf_token in map_clients)

        # Check if cached connection map has the correct
        # OneViewIp/OneViewConnection tuples
        for ov_ip, ov_conn in map_clients[rf_token].items():
            self.assertEqual(connections_ov[ov_ip], ov_conn)

    @mock.patch.object(connection, 'OneViewClient')
    @mock.patch.object(config, 'get_oneview_multiple_ips')
    @mock.patch.object(config, 'get_authentication_mode')
    def test_login_with_specific_login_domain_for_multiple_ov(
            self, get_authentication_mode,
            get_oneview_multiple_ips, oneview_client_mockup):

        get_authentication_mode.return_value = 'session'
        tokens_ov = collections.OrderedDict({'10.0.0.1': 'abc',
                                             '10.0.0.2': 'def',
                                             '10.0.0.3': 'ghi'})
        list_ips = list(tokens_ov.keys())

        client_session.init_map_clients()

        get_oneview_multiple_ips.return_value = list_ips

        client_session.login('SOME_DOMAIN\\user', 'password123')

        oneview_client_mockup.assert_any_call(
            {
                'ip': '10.0.0.1',
                'credentials': {
                    'userName': 'user',
                    'password': 'password123',
                    'authLoginDomain': 'SOME_DOMAIN'
                },
                'api_version': 600
            }
        )

    def test_create_credentials(self):
        # when username and password are simple values
        result = connection.create_credentials('administrator', 'pwd123')
        self.assertEqual(result, {
            'userName': 'administrator',
            'password': 'pwd123'
        })

        # when username has auth login domain
        result = connection.create_credentials('LOCAL\\administrator',
                                               'pwd123')
        self.assertEqual(result, {
            'userName': 'administrator',
            'password': 'pwd123',
            'authLoginDomain': 'LOCAL'
        })

        # when username and password are empty values
        result = connection.create_credentials('', '')
        self.assertEqual(result, {
            'userName': '',
            'password': ''
        })

        # when username and password are None
        result = connection.create_credentials(None, None)
        self.assertEqual(result, {
            'userName': None,
            'password': None
        })

    @mock.patch.object(client_session, 'uuid')
    @mock.patch.object(client_session, 'time')
    @mock.patch.object(client_session, 'threading')
    def test_garbage_collector_for_expired_sessions(self,
                                                    threading_mock,
                                                    time_mock,
                                                    uuid_mock):
        thread_obj_mock = mock.Mock()
        threading_mock.Thread.return_value = thread_obj_mock

        # using time.sleep to exit from loop raising an InterruptedError
        time_mock.sleep.side_effect = [None, InterruptedError]

        # Client 1 will represents the success request, must be in the cache
        # Client 2 will represents the fail request, must be removed from
        # the cache

        client_1 = mock.Mock()
        client_2 = mock.Mock()
        resp_1 = mock.Mock(status=200)
        resp_2 = mock.Mock(status=404)
        uuid_mock.uuid4.side_effect = ['session_id_1', 'session_id_2']

        client_1.connection.get_session_id.return_value = 'ov_session_abc'
        client_2.connection.get_session_id.return_value = 'ov_session_def'

        client_1.connection.do_http.return_value = (resp_1, None)
        client_2.connection.do_http.return_value = (resp_2, None)

        client_session._set_new_client_by_token('abc', {'10.0.0.11': client_1})
        client_session._set_new_client_by_token('def', {'10.0.0.12': client_2})

        expected_map_client = OrderedDict()
        expected_map_client['abc'] = {
            'client_ov_by_ip': {'10.0.0.11': client_1},
            'session_id': 'session_id_1'
        }

        client_session.init_gc_for_expired_sessions()
        try:
            client_session._gc_for_expired_sessions()
        except InterruptedError:
            pass

        self.assertEqual(expected_map_client,
                         client_session._get_map_clients())

        threading_mock.Thread.assert_called_with(
            target=client_session._gc_for_expired_sessions,
            daemon=True)
        thread_obj_mock.start.assert_called_with()
        time_mock.sleep.assert_called_with(client_session.GC_FREQUENCY_IN_SEC)

        client_1.connection.do_http.assert_called_with('GET',
                                                       '/rest/sessions/',
                                                       '',
                                                       {'Session-Id':
                                                        'ov_session_abc'})
        client_2.connection.do_http.assert_called_with('GET',
                                                       '/rest/sessions/',
                                                       '',
                                                       {'Session-Id':
                                                        'ov_session_def'})

    @mock.patch.object(client_session, 'time')
    def test_garbage_collector_loop(self, time_mock):
        time_mock.sleep.side_effect = [None, None, None, InterruptedError]

        try:
            client_session._gc_for_expired_sessions()
        except InterruptedError:
            pass

        time_mock.sleep.assert_called_with(client_session.GC_FREQUENCY_IN_SEC)
        self.assertEqual(4, time_mock.sleep.call_count)

    @mock.patch.object(client_session, 'uuid')
    @mock.patch.object(client_session, 'logging')
    @mock.patch.object(client_session, 'time')
    def test_garbage_collector_for_expired_sessions_when_raises_exception(
            self, time_mock, logging_mock, uuid_mock):
        time_mock.sleep.side_effect = [None, InterruptedError]

        uuid_mock.uuid4.return_value = 'session_id_1'

        client = mock.Mock()
        client.connection.do_http.side_effect = Exception('Some error message')
        client_session._set_new_client_by_token('abc', {'10.0.0.11': client})

        expected_map_client = OrderedDict()
        expected_map_client['abc'] = {
            'client_ov_by_ip': {'10.0.0.11': client},
            'session_id': 'session_id_1'
        }

        try:
            client_session._gc_for_expired_sessions()
        except InterruptedError:
            pass

        logging_mock.exception.assert_called_with(
            'Unexpected error: Some error message')
        self.assertEqual(expected_map_client,
                         client_session._get_map_clients())

    @mock.patch.object(client_session, 'uuid')
    @mock.patch.object(client_session, 'logging')
    @mock.patch.object(client_session, 'time')
    def test_garbage_collector_when_ov_request_status_is_not_200_neither_404(
            self, time_mock, logging_mock, uuid_mock):
        time_mock.sleep.side_effect = [None, None, None, None,
                                       InterruptedError]

        client = mock.Mock()
        client.connection.do_http.side_effect = [
            (mock.Mock(status=400), 'request body 1'),
            (mock.Mock(status=401), 'request body 2'),
            (mock.Mock(status=403), 'request body 3'),
            (mock.Mock(status=500), 'request body 4')
        ]
        uuid_mock.uuid4.return_value = 'session_id_1'
        client_session._set_new_client_by_token('abc', {'10.0.0.11': client})

        expected_map_client = OrderedDict()
        expected_map_client['abc'] = {
            'client_ov_by_ip': {'10.0.0.11': client},
            'session_id': 'session_id_1'
        }

        try:
            client_session._gc_for_expired_sessions()
        except InterruptedError:
            pass

        logging_mock.error.assert_has_calls([
            call(
                'Unexpected response with status 400 of Oneview sessions '
                'endpoint: request body 1'
            ),
            call(
                'Unexpected response with status 401 of Oneview sessions '
                'endpoint: request body 2'
            ),
            call(
                'Unexpected response with status 403 of Oneview sessions '
                'endpoint: request body 3'
            ),
            call(
                'Unexpected response with status 500 of Oneview sessions '
                'endpoint: request body 4'
            )
        ])
        self.assertEqual(expected_map_client,
                         client_session._get_map_clients())



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    Tests for test_subscription.py
"""

import json

from oneview_redfish_toolkit.api.subscription \
    import Subscription
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestSubscription(BaseTest):
    """Tests for Subscription class"""

    def setUp(self):
        """Tests preparation"""

        # Loading Subscription result mockup
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'Subscription.json'
        ) as f:
            self.subscription_mockup = json.load(f)

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated

        try:
            subscription = Subscription("", "", [], None)
        except Exception as e:
            self.fail("Failed to instantiate Subscription class."
                      " Error: {}".format(e))
        self.assertIsInstance(subscription, Subscription)

    def test_serialize(self):
        # Tests the serialize function result against known result

        try:
            subscription = Subscription(
                "e7f93fa2-0cb4-11e8-9060-e839359bc36a",
                "http://www.dnsname.com/Destination1",
                ["Alert", "ResourceUpdated"], None)
        except Exception as e:
            self.fail("Failed to instantiate Subscription class."
                      " Error: {}".format(e))

        try:
            result = json.loads(subscription.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: ".format(e))

        self.assertEqualMockup(self.subscription_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from collections import OrderedDict
import copy
import json
from unittest import mock

# 3rd party libs
from unittest.mock import call

from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
import oneview_redfish_toolkit.api.status_mapping as status_mapping
from oneview_redfish_toolkit.blueprints import resource_block
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestResourceBlock(BaseFlaskTest):
    """Tests for ResourceBlock blueprint"""

    @classmethod
    def setUpClass(self):
        super(TestResourceBlock, self).setUpClass()

        self.app.register_blueprint(resource_block.resource_block)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/ServerProfileTemplate.json'
        ) as f:
            self.server_profile_template = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/Drive.json'
        ) as f:
            self.drive = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/DriveIndexTrees.json'
        ) as f:
            self.drive_index_tree = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/'
            'DriveComposedIndexTrees.json'
        ) as f:
            self.drive_composed_index_tree = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview'
            '/ServerProfileTemplates.json'
        ) as f:
            self.server_profile_templates = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/LogicalEnclosures.json'
        ) as f:
            self.log_encl_list = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/redfish'
                '/ServerHardwareResourceBlock.json'
        ) as f:
            self.expected_sh_resource_block = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview'
                '/DriveEnclosureList.json'
        ) as f:
            self.drive_enclosure_list = json.load(f)

        self.resource_not_found = HPOneViewException({
            "errorCode": "RESOURCE_NOT_FOUND",
            "message": "Any resource not found message"
        })

    def test_get_resource_block_not_found(self):
        self.oneview_client.server_hardware.get.side_effect = \
            self.resource_not_found
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.resource_not_found
        self.oneview_client.index_resources.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/30303437-3034-4D32-3230-313133364752")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_storage_resource_block(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish/StorageResourceBlock.json'
        ) as f:
            expected_resource_block = json.load(f)

        self.oneview_client.server_hardware.get.side_effect = \
            self.resource_not_found
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.resource_not_found
        self.oneview_client.index_resources.get.return_value = self.drive
        self.oneview_client.connection.get.side_effect = [
            self.drive_index_tree,
        ]
        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.\
            logical_enclosures.get_all.return_value = self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        self.oneview_client.index_resources.get.assert_called_with(
            self.drive["uri"])
        self.oneview_client.connection.get.assert_has_calls(
            [
                call("/rest/index/trees/rest/drives/"
                     "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e?parentDepth=3"),
            ])
        self.oneview_client.\
            server_profile_templates.get_all.assert_called_with()
        self.oneview_client.logical_enclosures.get_all.assert_called_with()
        self.oneview_client.drive_enclosures.get_all.assert_called_with()

    def test_get_storage_resource_block_cached(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish/StorageResourceBlock.json'
        ) as f:
            expected_resource_block = json.load(f)

        self.oneview_client.server_hardware.get.side_effect = \
            self.resource_not_found
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.resource_not_found
        self.oneview_client.index_resources.get.return_value = self.drive
        self.oneview_client.connection.get.side_effect = [
            self.drive_index_tree, self.drive_index_tree,
        ]
        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.\
            logical_enclosures.get_all.return_value = self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        uri = "/redfish/v1/CompositionService/ResourceBlocks/"\
              "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e"
        uuid = uri.split('/')[-1]

        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        # Get cached storage resource block
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        self.oneview_client.server_profile_templates.get_all.has_calls(
            [call(uri),
             call(uri)]
        )
        self.oneview_client.server_hardware.get.assert_called_once_with(uuid)
        self.oneview_client.index_resources.get.assert_has_calls(
            [call('/rest/drives/' + uuid),
             call('/rest/drives/' + uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(
            ('/rest/drives/' + uuid)))

    def test_get_storage_resource_block_when_drive_is_composed(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish/'
            'StorageResourceBlockComposed.json'
        ) as f:
            expected_resource_block = json.load(f)

        drive_composed = copy.copy(self.drive)
        drive_composed["attributes"]["available"] = "no"

        self.oneview_client.server_hardware.get.side_effect = \
            self.resource_not_found
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.resource_not_found
        self.oneview_client.index_resources.get.return_value = drive_composed
        self.oneview_client.connection.get.side_effect = [
            self.drive_composed_index_tree,
        ]
        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        self.oneview_client.index_resources.get.assert_called_with(
            self.drive["uri"])
        self.oneview_client.connection.get.assert_has_calls(
            [
                call("/rest/index/trees/rest/drives/"
                     "c4f0392d-fae9-4c2e-a2e6-b22e6bb7533e?parentDepth=3"),
            ])
        self.oneview_client.\
            server_profile_templates.get_all.assert_called_with()
        self.oneview_client.logical_enclosures.get_all.assert_called_with()
        self.oneview_client.drive_enclosures.get_all.assert_called_with()

    def test_get_server_hardware_resource_block(self):
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware
        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/30303437-3034-4D32-3230-313133364752")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(self.expected_sh_resource_block, result)

    def test_all_server_hardware_resouce_block_states_with_sp(self):
        server_hardware = copy.deepcopy(self.server_hardware)
        expected_rb = copy.deepcopy(self.expected_sh_resource_block)
        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        for oneview_state, redfish_state in status_mapping.\
                SERVER_HARDWARE_STATE_TO_REDFISH_STATE.items():

            server_hardware["state"] = oneview_state
            expected_rb["Status"]["State"] = redfish_state
            expected_composition_state = status_mapping.\
                COMPOSITION_STATE.get(oneview_state)

            if not expected_composition_state:
                expected_composition_state = \
                    status_mapping.COMPOSITION_STATE["ProfileApplied"]

            expected_rb["CompositionStatus"]["CompositionState"] = \
                expected_composition_state

            self.oneview_client.server_hardware.get.return_value = \
                server_hardware

            response = self.client.get(
                "/redfish/v1/CompositionService/ResourceBlocks"
                "/30303437-3034-4D32-3230-313133364752")

            result = json.loads(response.data.decode("utf-8"))

            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqualMockup(expected_rb, result)

    def test_all_server_hardware_resouce_block_states_without_sp(self):
        server_hardware = copy.deepcopy(self.server_hardware)
        server_hardware["serverProfileUri"] = None
        expected_rb = copy.deepcopy(self.expected_sh_resource_block)
        expected_rb["Links"].pop("ComputerSystems")
        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        for oneview_state, redfish_state in status_mapping.\
                SERVER_HARDWARE_STATE_TO_REDFISH_STATE.items():

            server_hardware["state"] = oneview_state
            expected_rb["Status"]["State"] = redfish_state

            expected_composition_state = status_mapping.\
                COMPOSITION_STATE.get(oneview_state)

            if not expected_composition_state:
                expected_composition_state = status_mapping.\
                    COMPOSITION_STATE["NoProfileApplied"]

            expected_rb["CompositionStatus"]["CompositionState"] = \
                expected_composition_state

            self.oneview_client.server_hardware.get.return_value = \
                server_hardware

            response = self.client.get(
                "/redfish/v1/CompositionService/ResourceBlocks"
                "/30303437-3034-4D32-3230-313133364752")

            result = json.loads(response.data.decode("utf-8"))

            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqualMockup(expected_rb, result)

    def test_all_server_hardware_resouce_block_health(self):
        server_hardware = copy.deepcopy(self.server_hardware)
        expected_cs = copy.deepcopy(self.expected_sh_resource_block)

        self.oneview_client.server_profile_templates.get_all.return_value = \
            self.server_profile_templates
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        for oneview_status, redfish_status in \
                status_mapping.HEALTH_STATE.items():
            server_hardware["status"] = oneview_status
            expected_cs["Status"]["Health"] = redfish_status

            self.oneview_client.server_hardware.get.return_value = \
                server_hardware

            response = self.client.get(
                "/redfish/v1/CompositionService/ResourceBlocks"
                "/30303437-3034-4D32-3230-313133364752")

            result = json.loads(response.data.decode("utf-8"))

            self.assertEqual(status.HTTP_200_OK, response.status_code)
            self.assertEqual("application/json", response.mimetype)
            self.assertEqualMockup(expected_cs, result)

        self.oneview_client.server_hardware.get.assert_called_with(
            self.server_hardware["uuid"])

        encl_group_uri = self.server_hardware["serverGroupUri"]
        sh_type_uri = self.server_hardware["serverHardwareTypeUri"]
        self.oneview_client.\
            server_profile_templates.get_all.assert_called_with(
                filter=["enclosureGroupUri='" + encl_group_uri + "'",
                        "serverHardwareTypeUri='" + sh_type_uri + "'"]
                )
        self.oneview_client.logical_enclosures.get_all.assert_called_with()
        self.oneview_client.drive_enclosures.get_all.assert_called_with()

    def test_get_spt_resource_block(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/ServerProfileTemplateResourceBlock.json'
        ) as f:
            expected_resource_block = json.load(f)

        self.oneview_client.server_hardware.get.side_effect = \
            self.resource_not_found
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        self.oneview_client.logical_enclosures.get_all.assert_called_with()
        self.oneview_client.drive_enclosures.get_all.assert_called_with()

    def test_get_spt_resource_block_cached(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/ServerProfileTemplateResourceBlock.json'
        ) as f:
            expected_resource_block = json.load(f)

        self.oneview_client.server_hardware.get.side_effect = \
            self.resource_not_found
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list
        self.oneview_client.drive_enclosures.get_all.return_value = \
            self.drive_enclosure_list

        uri = "/redfish/v1/CompositionService/ResourceBlocks"\
              "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
        uuid = uri.split('/')[-1]

        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        # Get cached spt resource block
        response = self.client.get(uri)

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        self.oneview_client.server_hardware.get.assert_called_once_with(uuid)
        self.oneview_client.server_profile_templates.get.assert_has_calls(
            [call(uuid),
             call(uuid)]
        )
        self.assertTrue(category_resource.get_category_by_resource_id(uuid))

    def test_get_spt_resource_when_template_has_not_valid_controller(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/SPTResourceBlockWithOnlyOneZone.json'
        ) as f:
            expected_resource_block = json.load(f)

        self.oneview_client.server_hardware.get.side_effect = \
            self.resource_not_found
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_templates[1]
        self.oneview_client.logical_enclosures.get_all.return_value = \
            self.log_encl_list

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/75871d70-789e-4cf9-8bc8-6f4d73193578")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_resource_block, result)

        self.oneview_client.connection.get.assert_not_called()
        self.oneview_client.logical_enclosures.get.assert_not_called()

    def test_get_computer_system_not_found(self):
        self.oneview_client.server_hardware.get.side_effect = \
            HPOneViewException({"errorCode": "RESOURCE_NOT_FOUND"})

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/30303437-3034-4D32-3230-313133364752/Systems/1")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    @mock.patch.object(multiple_oneview, 'get_map_resources')
    @mock.patch.object(multiple_oneview, 'get_map_appliances')
    def test_get_computer_system(self, get_map_appliances, get_map_resources):
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/ResourceBlockComputerSystem.json'
        ) as f:
            expected_computer_system = json.load(f)

        # Loading ApplianceNodeInfo mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ApplianceNodeInfo.json'
        ) as f:
            appliance_info = json.load(f)

        map_appliance = OrderedDict({
            "10.0.0.1": appliance_info["uuid"]
        })

        get_map_resources.return_value = OrderedDict({
            "30303437-3034-4D32-3230-313133364752": "10.0.0.1",
        })
        get_map_appliances.return_value = map_appliance
        self.oneview_client.server_hardware.get.return_value = \
            self.server_hardware

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/30303437-3034-4D32-3230-313133364752/Systems/1")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_computer_system, result)

    def test_get_ethernet_interface(self):
        with open(
            'oneview_redfish_toolkit/mockups/redfish'
            '/ResourceBlockEthernetInterface.json'
        ) as f:
            expected_ethernet_interface = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/oneview/EthernetNetwork.json'
        ) as f:
            ethernet_network = json.load(f)

        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template
        self.oneview_client.index_resources.get.return_value = \
            ethernet_network

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0/EthernetInterfaces/1")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertEqualMockup(expected_ethernet_interface, result)

    def test_get_ethernet_interface_not_found(self):
        self.oneview_client.server_profile_templates.get.side_effect = \
            self.resource_not_found

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0/EthernetInterfaces/1")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)

    def test_get_ethernet_interface_invalid_id(self):
        self.oneview_client.server_profile_templates.get.return_value = \
            self.server_profile_template

        response = self.client.get(
            "/redfish/v1/CompositionService/ResourceBlocks"
            "/1f0ca9ef-7f81-45e3-9d64-341b46cf87e0/EthernetInterfaces/999")

        self.assertEqual(status.HTTP_404_NOT_FOUND, response.status_code)
        self.assertEqual("application/json", response.mimetype)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.errors import OneViewRedfishError
from oneview_redfish_toolkit.api.util.power_option import OneViewPowerOption
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestPowerOption(BaseTest):
    """Tests for Power Option class"""

    def setUp(self):

        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

    def test_power_option_valid_reset_actions(self):
        """Tests to get OneView Reset Actions with valid keys"""

        expected_oneview_power_options = {
            "On": {
                "powerState": "On",
            },
            "ForceOff": {
                "powerState": "Off",
                "powerControl": "PressAndHold"
            },
            "GracefulShutdown": {
                "powerState": "Off",
                "powerControl": "MomentaryPress"
            },
            "GracefulRestart": {
                "powerState": "On",
                "powerControl": "Reset"
            },
            "ForceRestart": {
                "powerState": "On",
                "powerControl": "ColdBoot"
            },
            "PushPowerButton": {
                "powerState": "Off",
                "powerControl": "MomentaryPress",
            }
        }

        reset_types = ["On", "ForceOff", "GracefulShutdown",
                       "GracefulRestart", "ForceRestart", "PushPowerButton"]

        for reset_type in reset_types:
            result = OneViewPowerOption.get_oneview_power_configuration(
                self.server_hardware, reset_type)
            self.assertEqual(
                result, expected_oneview_power_options[reset_type])

    def test_power_option_invalid_reset_actions(self):
        """Tests raises OneviewRedFishError exception for invalid keys"""

        reset_types = ["ForceOn", "Nmi"]

        for reset_type in reset_types:
            self.assertRaises(
                OneViewRedfishError,
                OneViewPowerOption.get_oneview_power_configuration,
                self.server_hardware, reset_type)

    def test_power_option_unmapped_reset_key(self):
        """Tests raises OneviewRedFishError exception for unmapped keys"""

        self.assertRaises(
            OneViewRedfishError,
            OneViewPowerOption.get_oneview_power_configuration,
            self.server_hardware, "INVALID_KEY")



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestComputerSystem(BaseTest):
    """Tests for ComputerSystem class"""

    def setUp(self):
        """Tests preparation"""

        # Loading server_hardware mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        # Loading ServerHardwareTypes mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardwareTypes.json'
        ) as f:
            self.server_hardware_types = json.load(f)

        # Loading ServerProfile mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerProfile.json'
        ) as f:
            self.server_profile = json.load(f)

        # Loading Drives mockup value
        with open(
                'oneview_redfish_toolkit/mockups/oneview/Drives.json'
        ) as f:
            self.drives = json.load(f)

        # Loading LabelForServerProfile mockup result
        with open(
                'oneview_redfish_toolkit/mockups/oneview'
                '/LabelForServerProfile.json'
        ) as f:
            self.label_for_server_profile = json.load(f)

        # Loading ComputerSystem mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/ComputerSystem.json'
        ) as f:
            self.computer_system_mockup = json.load(f)

        self.manager_uuid = "b08eb206-a904-46cf-9172-dcdff2fa9639"

    def test_serialize(self):
        # Tests the serialize function result against known result
        spt_uuid = "61c3a463-1355-4c68-a4e3-4f08c322af1b"
        computer_system = ComputerSystem(
            self.server_hardware,
            self.server_hardware_types,
            self.server_profile,
            [self.drives[4]],
            spt_uuid,
            self.manager_uuid
        )

        result = json.loads(computer_system.serialize())

        self.assertEqualMockup(self.computer_system_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import unittest

import oneview_redfish_toolkit.api.status_mapping as status_mapping


class TestStatusMapping(unittest.TestCase):
    """Tests for StatusMapping class"""

    def test_mapping(self):
        """Test mapping OneView Status to Redfish Status.

            In this case self.server_hardware["status"] is equal to OK.
        """

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            server_hardware = json.load(f)

        redfish_state, redfish_health = status_mapping.\
            get_redfish_server_hardware_status_struct(server_hardware)

        self.assertEqual(redfish_state, "Enabled")
        self.assertEqual(redfish_health, "OK")



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import copy
import json

# 3rd party libs
from unittest import mock
from unittest.mock import call

from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Module libs
from oneview_redfish_toolkit.blueprints import computer_system
from oneview_redfish_toolkit.services import computer_system_service
from oneview_redfish_toolkit.tests.base_flask_test import BaseFlaskTest


class TestCreateComputerSystem(BaseFlaskTest):
    """Tests for Create a ComputerSystem blueprint endpoint"""

    @classmethod
    def setUpClass(self):
        super(TestCreateComputerSystem, self).setUpClass()

        self.app.register_blueprint(computer_system.computer_system)

        self.not_found_error = HPOneViewException({
            'errorCode': 'RESOURCE_NOT_FOUND',
            'message': 'resource not found',
        })

        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerProfile.json'
        ) as f:
            self.server_profile = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)
            self.server_hardware['serverProfileUri'] = None

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'PostToComposeSystem.json'
        ) as f:
            self.data_to_create_system = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileTemplate.json'
        ) as f:
            self.server_profile_template = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'Drives.json'
        ) as f:
            self.drives = json.load(f)

        self.sh_id = "30303437-3034-4D32-3230-313133364752"
        self.spt_id = "1f0ca9ef-7f81-45e3-9d64-341b46cf87e0"
        self.drive1_id = "e11dd3e0-78cd-47e8-bacb-9813f4bb58a8"
        self.drive2_id = "53bd734f-19fe-42fe-a8ef-3f1a83b4e5c1"

        self.common_calls_to_assert_hardware = [
            call(self.sh_id),
            call(self.spt_id),
            call(self.drive1_id),
            call(self.drive2_id)
        ]

        self.common_calls_to_assert_spt = [
            call(self.sh_id),
            call(self.spt_id),
            call(self.drive1_id),
            call(self.drive2_id),
            call(self.spt_id)
        ]

        self.common_calls_to_assert_drives = [
            call('/rest/drives/' + self.sh_id),
            call('/rest/drives/' + self.spt_id),
            call('/rest/drives/' + self.drive1_id),
            call('/rest/drives/' + self.drive2_id)
        ]

    def run_common_mock_to_server_hardware(self):
        ov_client = self.oneview_client
        ov_client.server_hardware.get.side_effect = [
            self.server_hardware,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error,
            self.server_hardware,  # for multiple oneview (power update status)
            self.server_hardware  # for multiple oneview (create profile)
        ]
        ov_client.server_hardware.update_power_state.return_value = None

    def run_common_mock_to_server_profile_template(self):
        self.oneview_client.server_profile_templates.get.side_effect = [
            self.not_found_error,
            self.server_profile_template,
            self.not_found_error,
            self.not_found_error,
            self.server_profile_template
        ]

    def run_common_mock_to_drives(self):
        self.oneview_client.index_resources.get.side_effect = [
            self.not_found_error,
            self.not_found_error,
            self.drives[0],
            self.drives[1]
        ]

    def assert_common_calls(self):
        self.oneview_client.server_hardware.get.assert_has_calls(
            self.common_calls_to_assert_hardware)
        self.oneview_client.server_profile_templates.get.assert_has_calls(
            self.common_calls_to_assert_spt)
        self.oneview_client.index_resources.get.assert_has_calls(
            self.common_calls_to_assert_drives)

    @mock.patch.object(computer_system_service, 'time')
    def test_create_system(self, time_mock):
        """Tests create a redfish System with Network, Storage and Server"""

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileBuiltFromTemplateToCreateASystem.json'
        ) as f:
            expected_server_profile_built = json.load(f)

        task_without_resource_uri = {
            "associatedResource": {
                "resourceUri": None
            },
            "uri": "/rest/tasks/123456"
        }

        task_with_resource_uri = {
            "associatedResource": {
                "resourceUri": self.server_profile["uri"]
            },
            "uri": "/rest/tasks/123456"
        }

        self.run_common_mock_to_server_hardware()
        self.run_common_mock_to_server_profile_template()
        self.run_common_mock_to_drives()

        # The connection.post return for /rest/server-profiles is a tuple
        self.oneview_client.connection.post.return_value = \
            (task_without_resource_uri, None)

        # The task will be requested 3 times in this case,
        # simulating the checking of resource uri
        self.oneview_client.tasks.get.side_effect = [
            task_without_resource_uri,
            task_without_resource_uri,
            task_with_resource_uri
        ]

        response = self.client.post(
            "/redfish/v1/Systems",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertIn(
            "/redfish/v1/Systems/" + self.server_profile["uuid"],
            response.headers["Location"]
        )

        self.oneview_client.server_hardware.get.assert_has_calls(
            self.common_calls_to_assert_hardware)
        self.oneview_client.server_profile_templates.get.assert_has_calls(
            self.common_calls_to_assert_spt)
        self.oneview_client.index_resources.get.assert_has_calls(
            self.common_calls_to_assert_drives)
        self.oneview_client.server_profiles.create.assert_not_called()

        self.oneview_client.server_hardware.update_power_state \
            .assert_called_with({
                'powerControl': 'PressAndHold', 'powerState': 'Off'
            }, self.server_hardware['uuid'])
        self.oneview_client.tasks.get.assert_called_with(
            task_without_resource_uri["uri"])
        self.oneview_client.connection.post.assert_called_once_with(
            '/rest/server-profiles', expected_server_profile_built
        )
        self.assertEqual(self.oneview_client.tasks.get.call_count, 3)
        time_mock.sleep.assert_called_with(3)

    def test_create_when_server_hardware_already_belongs_to_system(self,):
        """Tests create when server profile already applied to the server"""
        sh_with_profile_uri = copy.deepcopy(self.server_hardware)
        sh_with_profile_uri["serverProfileUri"] = "/server-profiles/uuid_1"

        self.oneview_client.server_hardware.get.side_effect = [
            sh_with_profile_uri,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error,
            sh_with_profile_uri,
            sh_with_profile_uri
        ]

        self.run_common_mock_to_server_profile_template()
        self.run_common_mock_to_drives()

        response = self.client.post(
            "/redfish/v1/Systems",
            data=json.dumps(self.data_to_create_system),
            content_type="application/json")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertIn("Computer System Resource Block already belongs to a "
                      "Composed System with ID uuid_1", str(result))

        self.oneview_client.connection.post.assert_not_called()

    @mock.patch.object(computer_system_service, 'config')
    def test_create_when_power_off_on_compose_is_not_configured(self,
                                                                config_mock):
        """Tests create when power_off is blank, should below a normal flow"""

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileBuiltFromTemplateToCreateASystem.json'
        ) as f:
            expected_server_profile_built = json.load(f)

        self.run_common_mock_to_server_hardware()
        self.run_common_mock_to_server_profile_template()
        self.run_common_mock_to_drives()

        config_mock.get_composition_settings.return_value = {
            'PowerOffServerOnCompose': ''
        }

        task_with_resource_uri = {
            "associatedResource": {
                "resourceUri": self.server_profile["uri"]
            },
            "uri": "/rest/tasks/123456"
        }

        self.oneview_client.connection.post.return_value = \
            (task_with_resource_uri, None)

        response = self.client.post(
            "/redfish/v1/Systems",
            data=json.dumps(self.data_to_create_system),
            content_type="application/json")

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)

        self.oneview_client.server_hardware.update_power_state \
            .assert_not_called()
        self.oneview_client.connection.post.assert_called_once_with(
            '/rest/server-profiles', expected_server_profile_built
        )

    @mock.patch.object(computer_system_service, 'config')
    def test_create_when_power_off_on_compose_has_wrong_configuration(
            self, config_mock):
        """Tests create when power_off is blank, should below a normal flow"""

        self.run_common_mock_to_server_hardware()
        self.run_common_mock_to_server_profile_template()
        self.run_common_mock_to_drives()

        config_mock.get_composition_settings.return_value = {
            'PowerOffServerOnCompose': 'ForceOffff'
        }

        response = self.client.post(
            "/redfish/v1/Systems",
            data=json.dumps(self.data_to_create_system),
            content_type="application/json")

        result = json.loads(response.data.decode("utf-8"))

        self.assertEqual(status.HTTP_403_FORBIDDEN, response.status_code)
        self.assertIn('There is no mapping for ForceOffff on the OneView',
                      str(result))

        self.oneview_client.server_hardware.update_power_state \
            .assert_not_called()
        self.oneview_client.connection.post.assert_not_called()

    @mock.patch.object(computer_system_service, 'time')
    def test_create_system_without_description(self, time_mock):
        """Tests create a redfish System with Network, Storage and Server"""

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileBuiltFromTemplateToCreateASystem.json'
        ) as f:
            expected_server_profile_built = json.load(f)

        task_without_resource_uri = {
            "associatedResource": {
                "resourceUri": None
            },
            "uri": "/rest/tasks/123456"
        }

        task_with_resource_uri = {
            "associatedResource": {
                "resourceUri": self.server_profile["uri"]
            },
            "uri": "/rest/tasks/123456"
        }

        data_to_create_without_desc = copy.deepcopy(self.data_to_create_system)
        del expected_server_profile_built['description']
        data_to_create_without_desc['Description'] = ''

        self.run_common_mock_to_server_hardware()
        self.run_common_mock_to_server_profile_template()
        self.run_common_mock_to_drives()

        # The connection.post return for /rest/server-profiles is a tuple
        self.oneview_client.connection.post.return_value = \
            (task_without_resource_uri, None)

        # The task will be requested 3 times in this case,
        # simulating the checking of resource uri
        self.oneview_client.tasks.get.side_effect = [
            task_without_resource_uri,
            task_without_resource_uri,
            task_with_resource_uri
        ]

        response = self.client.post(
            "/redfish/v1/Systems",
            data=json.dumps(data_to_create_without_desc),
            content_type='application/json')

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertIn(
            "/redfish/v1/Systems/" + self.server_profile["uuid"],
            response.headers["Location"]
        )

        self.oneview_client.server_hardware.get.assert_has_calls(
            self.common_calls_to_assert_hardware)
        self.oneview_client.server_profile_templates.get.assert_has_calls(
            self.common_calls_to_assert_spt)
        self.oneview_client.index_resources.get.assert_has_calls(
            self.common_calls_to_assert_drives)
        self.oneview_client.server_profiles.create.assert_not_called()

        self.oneview_client.tasks.get.assert_called_with(
            task_without_resource_uri["uri"])
        self.oneview_client.connection.post.assert_called_once_with(
            '/rest/server-profiles', expected_server_profile_built
        )
        self.assertEqual(self.oneview_client.tasks.get.call_count, 3)
        time_mock.sleep.assert_called_with(3)

    def test_create_system_when_request_content_is_wrong(self):
        """Tests trying create a redfish System without Links"""

        data_to_send = {
            "Name": "Composed System Without Links"
        }

        response = self.client.post("/redfish/v1/Systems",
                                    data=json.dumps(data_to_send),
                                    content_type='application/json')

        self.assertEqual(
            status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_hardware.get.assert_not_called()
        self.oneview_client.server_profile_templates.get.assert_not_called()
        self.oneview_client.index_resources.get.assert_not_called()
        self.oneview_client.connection.post.assert_not_called()

    def test_create_system_when_request_content_has_not_compute(self):
        """Tests trying create a redfish System without Compute Resource"""

        self.oneview_client.server_hardware.get.side_effect = [
            self.not_found_error,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error
        ]

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertIn(
            "Should have a Computer System Resource Block",
            response.data.decode()
        )
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_hardware.get.assert_has_calls(
            self.common_calls_to_assert_hardware)
        self.oneview_client.server_profile_templates.get.assert_not_called()
        self.oneview_client.index_resources.get.assert_not_called()
        self.oneview_client.connection.post.assert_not_called()

    def test_create_system_when_request_content_has_not_network(self):
        """Tests trying create a redfish System without Network Resource"""

        self.oneview_client.server_hardware.get.side_effect = [
            self.server_hardware,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error
        ]

        self.oneview_client.server_profile_templates.get.side_effect = [
            self.not_found_error,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error
        ]

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertIn(
            "Should have a valid Network Resource Block",
            response.data.decode()
        )
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_hardware.get.assert_has_calls(
            self.common_calls_to_assert_hardware)
        self.oneview_client.server_profile_templates.get.assert_has_calls(
            [
                call(self.sh_id),
                call(self.spt_id),
                call(self.drive1_id),
                call(self.drive2_id)
            ])
        self.oneview_client.index_resources.get.assert_not_called()
        self.oneview_client.connection.post.assert_not_called()

    def test_create_system_when_request_content_has_not_a_valid_network(
            self):
        """Tests trying create a redfish System with a invalid Network"""

        self.oneview_client.server_hardware.get.side_effect = [
            self.server_hardware,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error
        ]

        self.oneview_client.server_profile_templates.get.side_effect = [
            self.not_found_error,
            self.server_profile_template,
            self.not_found_error,
            self.not_found_error
        ]

        data_to_send = copy.copy(self.data_to_create_system)
        # wrong SPT id
        data_to_send["Id"] = "75871d70-789e-4cf9-8bc8-6f4d73193578"

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(data_to_send),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertIn(
            "Should have a valid Network Resource Block",
            response.data.decode()
        )
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_hardware.get.assert_has_calls(
            self.common_calls_to_assert_hardware)
        self.oneview_client.server_profile_templates.get.assert_has_calls(
            [
                call(self.sh_id),
                call(self.spt_id),
                call(self.drive1_id),
                call(self.drive2_id)
            ])
        self.oneview_client.index_resources.get.assert_not_called()
        self.oneview_client.connection.post.assert_not_called()

    @mock.patch.object(computer_system_service, 'time')
    def test_create_system_when_request_content_has_not_storage(self, _):
        """Tests create a redfish System without Storage.

            This test should works well.

            This case is when we are creating a System without Storage
            Resource Blocks, but the Server Profile Template related has a
            local storage controller configured properly
        """

        with open(
                'oneview_redfish_toolkit/mockups/redfish/'
                'PostToComposeSystemWithoutStorage.json'
        ) as f:
            data_to_send = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileBuiltFromZoneWithoutStorageToCreateASystem.json'
        ) as f:
            expected_server_profile_built = json.load(f)

        with open(
                'oneview_redfish_toolkit/mockups/oneview/'
                'ServerProfileTemplates.json'
        ) as f:
            list_spt = json.load(f)
            spt = list_spt[1]  # without local storage controller configured
            spt_id = spt["uri"].split("/")[-1]

        task = {
            "associatedResource": {
                "resourceUri": self.server_profile["uri"]
            },
            "uri": "/rest/tasks/123456"
        }

        self.oneview_client.server_hardware.get.side_effect = [
            self.server_hardware,
            self.not_found_error,
            self.server_hardware,  # for multiple oneview (power update status)
            self.server_hardware
        ]
        self.oneview_client.server_profile_templates.get.side_effect = [
            self.not_found_error,
            spt,
            spt,
        ]
        self.oneview_client.index_resources.get.side_effect = [
            self.not_found_error,
            self.not_found_error,
        ]
        self.oneview_client.connection.post.return_value = (task, None)

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(data_to_send),
            content_type='application/json')

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertIn(
            "/redfish/v1/Systems/" + self.server_profile["uuid"],
            response.headers["Location"]
        )
        self.oneview_client.server_hardware.get.assert_has_calls(
            [
                call(self.sh_id),
                call(spt_id),
            ])
        self.oneview_client.server_profile_templates.get.assert_has_calls(
            [
                call(self.sh_id),
                call(spt_id),
                call(spt_id)
            ])
        self.oneview_client.index_resources.get.assert_has_calls(
            [
                call('/rest/drives/' + self.sh_id),
                call('/rest/drives/' + spt_id),
            ])

        self.oneview_client.connection.post.assert_called_with(
            '/rest/server-profiles',
            expected_server_profile_built)

    @mock.patch.object(computer_system_service, 'time')
    def test_create_system_when_has_not_storage_and_controller(self, _):
        """Tests create a System without Storage but with Storage Controller.

            This test should works well.

            This case is when we are creating a System without Storage
            Resource Blocks and the Server Profile Template related has not a
            local storage controller configured properly
        """

        task = {
            "associatedResource": {
                "resourceUri": self.server_profile["uri"]
            },
            "uri": "/rest/tasks/123456"
        }

        self.oneview_client.server_hardware.get.side_effect = [
            self.server_hardware,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error,
            self.server_hardware,  # for multiple oneview (power update status)
            self.server_hardware  # Get for multiple OneView support
        ]

        template_without_controller = copy.deepcopy(
            self.server_profile_template)
        template_without_controller["localStorage"]["controllers"] = []
        self.oneview_client.server_profile_templates.get.side_effect = [
            self.not_found_error,
            template_without_controller,
            self.not_found_error,
            self.not_found_error,
            template_without_controller
        ]
        self.oneview_client.index_resources.get.side_effect = [
            self.not_found_error,
            self.not_found_error,
            self.not_found_error,
            self.not_found_error
        ]
        self.oneview_client.connection.post.return_value = (task, None)

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.assertIn(
            "/redfish/v1/Systems/" + self.server_profile["uuid"],
            response.headers["Location"]
        )
        self.assert_common_calls()

    def test_create_system_when_a_generic_exception_is_raised(self):
        """Tests create a redfish System when occurs a generic exception"""

        self.oneview_client.server_hardware.get.side_effect = Exception()

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        self.assertEqual(status.HTTP_500_INTERNAL_SERVER_ERROR,
                         response.status_code)
        self.assertEqual("application/json", response.mimetype)
        self.oneview_client.server_hardware.get.assert_called_with(self.sh_id)
        self.oneview_client.server_profile_templates.get.assert_not_called()
        self.oneview_client.index_resources.get.assert_not_called()

    @mock.patch.object(computer_system_service, 'time')
    def test_create_system_when_a_task_error_is_raised(self, _):
        """Tests create a System when the Oneview raises a task error.

            This test should return a http 403 with a error message.
            Some problems are server hardware is powered On and the drive used
            belongs to another enclosure.
        """

        task = {
            "associatedResource": {
                "resourceUri": None
            },
            "uri": "/rest/tasks/123456",
            "taskErrors": [
                {"message": "The server hardware 123 is powered on"}
            ]
        }

        self.run_common_mock_to_server_hardware()
        self.run_common_mock_to_server_profile_template()
        self.run_common_mock_to_drives()

        self.oneview_client.connection.post.return_value = (task, object())

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_403_FORBIDDEN,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.assertIn("The server hardware 123 is powered on",
                      response.data.decode())

        self.assert_common_calls()

    @mock.patch.object(computer_system_service, 'time')
    def test_when_has_more_than_one_task_error(self, _):
        """Tests create a System when the Oneview raises two task errors.

            This test should return a http 403 with a error message.
            Some problems are server hardware is powered On and the drive used
            belongs to another enclosure.
        """

        task = {
            "associatedResource": {
                "resourceUri": None
            },
            "uri": "/rest/tasks/123456",
            "taskErrors": [
                {"message": "The server hardware 123 is powered on"},
                {"message": "The drive used belongs to another enclosure"}
            ]
        }

        self.run_common_mock_to_server_hardware()
        self.run_common_mock_to_server_profile_template()
        self.run_common_mock_to_drives()

        self.oneview_client.connection.post.return_value = (task, object())

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        expected_error_msg = "The server hardware 123 is powered on\\n" \
                             "The drive used belongs to another enclosure\\n"

        self.assertEqual(
            status.HTTP_403_FORBIDDEN,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.assertIn(expected_error_msg, response.data.decode())

        self.assert_common_calls()

    def test_create_system_when_has_storage_but_not_valid_controller(self):
        """Tests when the Server Profile Template has not a valid storage controller.

            This test should return a http 403 with a error message.

            The case is: the server profile template associated with the
            request has not a valid local storage controller configured,
            but the request has storage resource blocks to compose the system
        """

        self.run_common_mock_to_server_hardware()

        template_without_controller = copy.deepcopy(
            self.server_profile_template)
        template_without_controller["localStorage"]["controllers"] = []
        self.oneview_client.server_profile_templates.get.side_effect = [
            self.not_found_error,
            template_without_controller,
            self.not_found_error,
            self.not_found_error,
            template_without_controller
        ]

        self.run_common_mock_to_drives()

        response = self.client.post(
            "/redfish/v1/Systems/",
            data=json.dumps(self.data_to_create_system),
            content_type='application/json')

        self.assertEqual(
            status.HTTP_403_FORBIDDEN,
            response.status_code
        )
        self.assertEqual("application/json", response.mimetype)
        self.assertIn("The Server Profile Template should have a valid "
                      "storage controller to use the Storage Resource "
                      "Blocks passed",
                      response.data.decode())

        self.assert_common_calls()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
from unittest import TestCase

from oneview_redfish_toolkit import config


class BaseTest(TestCase):
    """Base test"""

    @classmethod
    def setUpClass(cls):
        patcher = mock.patch('oneview_redfish_toolkit.connection.'
                             'check_oneview_availability')
        cls.mock_check_availability = patcher.start()

        cls.config_file = './oneview_redfish_toolkit/conf/redfish.conf'

        config.load_config(cls.config_file)

    def assertEqualMockup(self, first, second, msg=None):
        if type(first) is dict and type(second) is dict:
            first['@odata.type'] = ''
            second['@odata.type'] = ''

        super(BaseTest, self).assertEqual(first, second, msg)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import json

# 3rd party libs

# Own project libs
from oneview_redfish_toolkit.api.rack_chassis import RackChassis
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestRackChassis(BaseTest):
    """Tests for Chassis class

        Tests:
            - Rack chassis instantiation
            - Rack chassis serialize
    """

    def setUp(self):
        """Tests preparation"""

        # Loading rack mockup value
        with open(
            'oneview_redfish_toolkit/mockups/oneview/Rack.json'
        ) as f:
            self.rack = json.load(f)

        # Loading rack_chassis_mockup mockup result
        with open(
            'oneview_redfish_toolkit/mockups/redfish/RackChassis.json'
        ) as f:
            self.rack_chassis_mockup = json.load(f)

    def test_class_instantiation(self):
        # Tests if class is correctly instantiated and validated

        try:
            rack_chassis = RackChassis(self.rack)
        except Exception as e:
            self.fail("Failed to instantiate RackChassis class."
                      " Error: {}".format(e))
        self.assertIsInstance(rack_chassis, RackChassis)

    def test_serialize(self):
        # Tests the serialize function result against known result

        try:
            rack_chassis = RackChassis(self.rack)
        except Exception as e:
            self.fail("Failed to instantiate RackChassis class."
                      " Error: {}".format(e))

        try:
            result = json.loads(rack_chassis.serialize())
        except Exception as e:
            self.fail("Failed to serialize. Error: ".format(e))

        self.assertEqualMockup(self.rack_chassis_mockup, result)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json

from oneview_redfish_toolkit.api.processor_collection \
    import ProcessorCollection
from oneview_redfish_toolkit.tests.base_test import BaseTest


class TestProcessorCollection(BaseTest):

    @classmethod
    def setUpClass(self):
        super(TestProcessorCollection, self).setUpClass()

        with open(
            'oneview_redfish_toolkit/mockups/oneview/ServerHardware.json'
        ) as f:
            self.server_hardware = json.load(f)

        with open(
            'oneview_redfish_toolkit/mockups/redfish/ProcessorCollection.json'
        ) as f:
            self.processor_collection_mockup = json.load(f)

    def test_serialize(self):
        processor_collection = ProcessorCollection(self.server_hardware)

        result = json.loads(processor_collection.serialize())

        self.assertEqualMockup(self.processor_collection_mockup, result)
